-- ExplosiveSpawner.lua --
obj_survivalobject_farmerball = sm.uuid.new( "8d601982-4608-4d5e-bb9e-e4041486f7c7" )

CagedFarmer = class()

CagedFarmer.maxParentCount = 1
CagedFarmer.maxChildCount = 0
CagedFarmer.connectionInput = sm.interactable.connectionType.logic
CagedFarmer.connectionOutput = sm.interactable.connectionType.none
CagedFarmer.colorNormal = sm.color.new( 0xcb0a00ff )
CagedFarmer.colorHighlight = sm.color.new( 0xee0a00ff )

--[[ Server ]]

-- (Event) Called upon creation on server
function CagedFarmer.server_onCreate( self )
	self.interactable.active = false
	self.spawnedObject = nil
    self.spawnTimer = 0
end

-- (Event) Called upon game tick. (40 times a second)
function CagedFarmer.server_onFixedUpdate( self, timeStep )

	--refresh input state
	local parent = self.interactable:getSingleParent()
	if parent then
		self.interactable.active = parent.active
	else
		self.interactable.active = false
	end
	
	if(self.spawnTimer < 0) then --wait 25 ticks if an explosive has just been created
		self.spawnTimer = self.spawnTimer + 1
	elseif self.interactable.active then --if signal is active
		self.spawnTimer = self.spawnTimer + 1
		if self.spawnTimer >= 10 then --wait 10 tics then
			--check if the new explosive can spawn
			if self.spawnedObject ~= nil and sm.exists( self.spawnedObject ) then -- No tank or tank was destroyed
				dist = (self.spawnedObject:getWorldPosition() - self.shape:getWorldPosition() - self.shape.at * 0.5)
				if sm.vec3.length(dist) < 0.45 then
					print("Small explosive spawner: Can't spawn the explosive, the previous one is too close")
					return --break out of the loop
				end
			end
			--determine position to spawn the explosive
			local spawnPos = self.shape:getWorldPosition() + ( self.shape.at * 1) - (self.shape.right * 1 + self.shape.up * 0.25) --spawn in the center, 0.5 block above

			
			self.spawnedObject = sm.shape.createPart( obj_survivalobject_farmerball, spawnPos, self.shape.worldRotation, true, true )
			self.spawnedObject.color = self.shape.color -- Blue spawners spawn propanetanks
                    
			self.spawnedObject.body.destructable = true
			self.spawnedObject.body.buildable = true
			self.spawnedObject.body.paintable = true
			self.spawnedObject.body.connectable = false 
			self.spawnedObject.body.liftable = true
			self.spawnedObject.body.erasable = true
                    
			self.network:sendToClients( "client_onTankSpawned" )
					
			self.spawnTimer = -25 --delay so 
			self.spawnedObject.body.destructable = true
		end
	else --inactive
		self.spawnTimer = 0
	end			
		
end

--[[ Client ]]

-- (Event) Called upon creation on client
function CagedFarmer.client_onCreate( self )
	self.activationEffect = sm.effect.createEffect( "Ballspawner - Activate", self.interactable )
end

-- (Event) Called upon through the network when creating a new ball
function CagedFarmer.client_onTankSpawned( self )
	self.activationEffect:start()
end