-- BotMaker.lua --

BotMaker = class()

BotMaker.maxParentCount = 1
BotMaker.maxChildCount = 0
BotMaker.connectionInput = sm.interactable.connectionType.logic
BotMaker.connectionOutput = sm.interactable.connectionType.none
BotMaker.colorNormal = sm.color.new( 0xccddccff )
BotMaker.colorHighlight = sm.color.new( 0xffeeffff )

BotMaker.parentActive = false;

--BotMaker.poseWeightCount = 1

-- [[ Server ]]

-- (Event) Called upon game tick. (40 times a second)
function BotMaker.server_onFixedUpdate( self, timeStep )
	self:server_tryUnpack()
	self.parentActive = false
	local parent = self.interactable:getSingleParent()
	if parent then
		self.parentActive = parent:isActive()
	end
end

-- Attempt to unpack the BotMaker
function BotMaker.server_tryUnpack( self )
	local parent = self.interactable:getSingleParent()
	if parent and not self.parentActive then
		if parent:isActive() and not self.parentActive then
			self:sv_unpack()
		end
	end
end

function BotMaker.sv_unpack( self )
	local yaw = math.atan2( self.shape.up.y, self.shape.up.x ) - math.pi / 2
	local zShapeOffset = math.abs( ( self.shape.worldRotation * sm.item.getShapeOffset( self.shape.uuid ) ).z )
	local spawnOffset = sm.vec3.new( 0, 0, zShapeOffset * 3 )
	sm.unit.createUnit( sm.uuid.new( self.data.unitUuid ), self.shape.worldPosition + spawnOffset, yaw, { color = self.shape.color } )
end

-- (Event) Called upon getting hit by a projectile.
function BotMaker.server_onProjectile( self, hitPos, hitTime, hitVelocity, projectileName, attacker, damage )
	self:sv_unpack()
end

-- (Event) Called upon getting hit by a melee attack.
function BotMaker.server_onMelee( self, hitPos, attacker, damage )
	self:sv_unpack()
end

-- (Event) Called upon collision with an explosion nearby
function BotMaker.server_onExplosion( self, center, destructionLevel )
	self:sv_unpack()
end
